/*****************************************************************************\
*              efs - General purpose Embedded Filesystem library              *
*          --------------------- -----------------------------------          *
*                                                                             *
* Filename : lpc2214spi0.c                                                    *
* Description : This file contains the functions needed to use efs for        *
*               accessing files on an SD-card connected to an LPC2xxx.        *
* Interface:  eigentlich SPI 0, aber die SPI-Peripherie funktioniert nicht,   *
*             also mache ich das Ganze per Software. zum Kotzen, aber...      *
\*****************************************************************************/

#include "STM32F103.h"
#include "lpc_spi.h"
#include "sd.h"
#include "config.h"
#include "error.h"
#include "serial.h"
#include "ioman.h"



/* Sektoren lesen und schreiben: wird einfach weitergeleitet. siehe sd.c  */

esint8 if_readBuf (euint32 address, euint8* buf)
{ return (sd_readSector(address,buf,512)); }


esint8 if_writeBuf (euint32 address, euint8* buf)
{ return (sd_writeSector(address, buf)); }



void Select_Card (void)
{ GPIOC->ODR &= ~(1<<SPI_SS_PIN);
}


void Unselect_Card (void)
{ GPIOC->ODR |= (1<<SPI_SS_PIN); }



/* hier erfolgt das eigentliche Data-In und Out ber SPI */

euint8 if_spiDIO (euint8 outgoing)
{ euint8 incoming;
  int i;

  incoming = 0;
  i = 8;
  while (i)
  { if (outgoing & 128)
      GPIOD->ODR |= (1<<SPI_MOSI_PIN);
    else
      GPIOD->ODR &= ~(1<<SPI_MOSI_PIN);
    GPIOC->ODR |= (1<<SPI_SCK_PIN);
    outgoing = outgoing<<1;
    incoming = (incoming<<1);
    if (GPIOC->IDR & (1<<SPI_MISO_PIN)) ++incoming;
    GPIOC->ODR &= ~(1<<SPI_SCK_PIN);
    --i;
  }
  return(incoming);
}



/* Interface initialisieren.
Das braucht es nach JEDEM neuen Einschieben einer Speicherkarte.
Vorbemerkungen:
 - die Pin-Funktionen sind bereits im Startupcode richtig aufgesetzt
   (eigentlich! wenn denn die SPI-Hardware funktionieren wrde...)
 - die Port-Funktion (In/Out) von 'Nichtfunktions'- also Port-pins
   sind ab Systemstart erstmal alle auf Input gesetzt und brauchen
   eine Initialisierung

 Initialisieren der SD-Karte:
   a) MOSI + CS high
   b) mindestens 74 x SCLK pulsen
   c) CS low (= aktiv = select)
   d) Kommando 0 applizieren: GO_IDLE_STATE
   e) Response (Typ=R1, entspricht 1 Byte mit Bit 7 = low) einholen
      (kann bis zu 8 Bytes dauern)
   f) CS high (= unselect)
   g) Kommando 1 applizieren: SEND_OP_COND

 */

esint8 if_initInterface (hwInterface *iface)
{ euint32 sc;
  esint16 i;
  euint16 resp;

  /* erstmal Handle austragen */
  iface->handle      = 0;
  SD_DriveSize = 0;

  Unselect_Card();

/* a */
  GPIOC->ODR |= (1<<SPI_MOSI_PIN);      /* MOSI auf high */
  GPIOC->ODR &= ~(1<<SPI_SCK_PIN);       /* SCK  auf low  */
  Unselect_Card();

/* b */
  i = 100;
  while (i)
  { GPIOC->ODR |= (1<<SPI_SCK_PIN);
    --i;
    GPIOC->ODR &= ~(1<<SPI_SCK_PIN);
  }

/* c + d + e + f */
  resp = sd_Command (0,0,0);               /* CMD0 = GO_IDLE_STATE */
  if(resp >= 0xff) return(SD_noresponse);
  if (resp != 1)   return(SD_noidle);

/* g */
  i=32000;
  do { resp = sd_Command(1, 0, 0);         /* huch? CMD1 = reserved ? */
     } while (resp==1 && i--);

  if(resp!=0)   return(SD_initfailed);

  i = sd_Command(13, 0, 0);                /* CMD13 = SEND_STATUS */
  if(i!=0)     return(SD_nostatus);

  sd_getDriveSize(&sc);                    /* ber CMD9 = SEND_CSD */
  SD_DriveSize = sc;

  iface->sectorCount = sc/512;
  iface->handle      = 1;
  if( (sc%512) != 0) iface->sectorCount--;
#ifdef DEBUG
  V24_StrOut("Drive Size is ");
  Long_Out(sc);
  V24_StrOut(" Bytes (");
  Long_Out(iface->sectorCount);
  V24_StrOut(" Sectors)\n");
  V24_StrOut("Init done...\n");
#endif
  return(0);
}
